package com.ludoko.ld48;

import flixel.effects.FlxTrail;
import flixel.FlxG;
import flixel.FlxSprite;
import flixel.util.FlxAngle;
import flixel.util.FlxMath;

/**
 * Bullet
 * 
 * @author Michael Lee
 */
class Bullet extends FlxSprite
{

	
	public var imgFile:String = "";
	public var radius:Int = 2;
	
	public function new(X:Float, Y:Float, VelX:Float = 0, VelY:Float = 0) 
	{
		super(X, Y);
		
		setupGraphic();
		resetBullet(X, Y, VelX, VelY);
	}
	
	private function setupGraphic():Void
	{
		loadGraphic("assets/images/game/bullet.png");
		imgFile = "assets/images/game/bullet.png";
		radius = 3;
	}
	
	public function resetBullet(X:Float, Y:Float, VelX:Float, VelY:Float):Void
	{
		reset(X - radius, Y - radius);
		velocity.x = _origVelX = VelX;
		velocity.y = _origVelY = VelY;
		maxVelocity.x = Math.abs(VelX);
		maxVelocity.y = Math.abs(VelY);
		_angVelocity = FlxMath.vectorLength(VelX, VelY);
		
		_angle = Math.atan2(VelY, VelX);
		
		_reverseVelX = -VelX;
		_reverseVelY = -VelY;
		
		_trail = G.playstate.addTrail(this, imgFile);
	}
	
	public var centerX(get_centerX, never):Float;
	public var centerY(get_centerY, never):Float;
	private function get_centerX():Float { return x + radius; }
	private function get_centerY():Float { return y + radius; }
	
	override public function update():Void 
	{
		var timeScale:Float = 1;
		
		G.p1.x = x - radius;
		G.p1.y = y - radius;
		
		var c:TimeCircle;
		for (c in G.playstate.circles)
		{
			G.p2.x = c.centerX;
			G.p2.y = c.centerY;
			var dist:Float = FlxMath.getDistance(G.p1, G.p2);
			
			var cscale:Float = c.getAdjustedTimeScale(dist);
			//if (cscale < timeScale)
			//{
				timeScale *= cscale;
			//}
		}
		
		if (timeScale != 1)
		{
			if (timeScale > 0)
			{
				var newVel:Float = _angVelocity * timeScale;
				if (!_reversed)
				{
					velocity.x = Math.cos(_angle) * newVel;
					velocity.y = Math.sin(_angle) * newVel;
				}
				else
				{
					velocity.x = -Math.cos(_angle) * newVel;
					velocity.y = -Math.sin(_angle) * newVel;
				}
			}
			else
			{
				acceleration.x = _origVelX * timeScale;
				acceleration.y = _origVelY * timeScale;
				_reversed = true;
			}
		}
		
		super.update();
		
		_timer -= FlxG.elapsed;
		if (_timer <= 0)
		{
			if (!onScreen())
			{
				kill();
			}
		}
	}
	
	override public function kill():Void 
	{
		super.kill();
		G.playstate.bullets.remove(this, true);
		G.playstate.trails.remove(_trail, true);
		_trail.kill();
		_trail = null;
	}
	
	private var _timer:Float = 1;
	
	private var _origVelX:Float;
	private var _origVelY:Float;
	
	private var _angVelocity:Float;
	private var _angle:Float;
	
	private var _reversed:Bool = false;
	private var _reverseVelX:Float;
	private var _reverseVelY:Float;
	
	private var _trail:FlxTrail;
	
}