package com.ludoko.ld48;

import flixel.FlxG;
import flixel.group.FlxGroup;
import flixel.text.FlxText;

/**
 * Game gui
 * 
 * @author Michael Lee
 */
class GameGUI extends FlxGroup
{
	
	public function new() 
	{
		super();
		
		var tx:Float = 130;
		var ty:Float = 4;
		_secs00 = setupTimeDigit(tx, ty);
		_secs0 = setupTimeDigit(tx + 18, ty);
		_frac00 = setupTimeDigit(tx + 43, ty);
		_frac0 = setupTimeDigit(tx + 61, ty);
		
		var dot:FlxText = new FlxText(tx + 41, ty, 30, ".");
		dot.setFormat(Assets.FNT_GUBBLABLO, 30, 0xdf50db);
		
		_total = new FlxText( -3, 6, FlxG.width, "0.00");
		_total.setFormat(Assets.FNT_GUBBLABLO, 20, 0xb7b8bd, "right");
		
		_last = new FlxText( -4, 30, FlxG.width, "0.00");
		_last.setFormat(Assets.FNT_GUBBLABLO, 10, 0xb7b8bd, "right");
		
		_best = new FlxText(3, 8, FlxG.width, "0.00");
		_best.setFormat(Assets.FNT_GUBBLABLO, 10, 0xb7b8bd, "left");
		
		var total:FlxText = new FlxText( -4, 1, FlxG.width, "TOTAL");
		total.setFormat(Assets.FNT_GUBBLABLO, 10, 0xc6c7cb, "right");
		
		var last:FlxText = new FlxText( -4, 23, FlxG.width, "LAST");
		last.setFormat(Assets.FNT_GUBBLABLO, 10, 0xc6c7cb, "right");
		
		var best:FlxText = new FlxText(3, 1, FlxG.width, "BEST");
		best.setFormat(Assets.FNT_GUBBLABLO, 10, 0xc6c7cb, "left");
		
		// Adds
		add(_secs00);
		add(_secs0);
		add(dot);
		add(_frac00);
		add(_frac0);
		
		add(total);
		add(last);
		add(best);
		add(_total);
		add(_last);
		add(_best);
	}
	
	private function setupTimeDigit(X:Float, Y:Float):FlxText
	{
		var t:FlxText = new FlxText(X, Y, 30, "0");
		t.setFormat(Assets.FNT_GUBBLABLO, 30, 0xdf50db, "center");
		return t;
	}
	
	private function setupTotalTimeDigit(X:Float, Y:Float):FlxText
	{
		var t:FlxText = new FlxText(X, Y, 30, "0");
		t.setFormat(Assets.FNT_GUBBLABLO, 30, 0x000000, "center");
		return t;
	}
	
	public function updateTime(Time:Float):Void
	{
		var t:String;
		
		var secs:Int = Math.floor(Time);
		if (secs < 0) secs = 0;
		if (secs < 10)
		{
			t = "0" + secs;
		}
		else
		{
			t = secs + "";
		}
		
		t += ".";
		
		Time -= secs;
		
		var f:Int = Math.floor(Time * 100);
		if (f < 0) f = 0;
		if (f < 10)
		{
			t += "0" + f;
		}
		else
		{
			t += f;
		}
		
		_secs00.text = t.substr(0, 1);
		_secs0.text = t.substr(1, 1);
		_frac00.text = t.substr(3, 1);
		_frac0.text = t.substr(4, 1);
	}
	
	public function updateTotalTime(Time:Float):Void
	{
		var t:String;
		var secs:Int = Math.floor(Time);
		t = secs + ".";
		
		Time -= secs;
		var f:Int = Math.floor(Time * 100);
		if (f < 0) f = 0;
		if (f < 10)
		{
			t += "0" + f;
		}
		else
		{
			t += f;
		}
		_total.text = t;
	}
	
	public function updateLastTime(Time:Float):Void
	{
		var t:String;
		var secs:Int = Math.floor(Time);
		t = secs + ".";
		
		Time -= secs;
		var f:Int = Math.floor(Time * 100);
		if (f < 0) f = 0;
		if (f < 10)
		{
			t += "0" + f;
		}
		else
		{
			t += f;
		}
		_last.text = t;
	}
	
	public function updateBestTime(Time:Float):Void
	{
		var t:String;
		var secs:Int = Math.floor(Time);
		t = secs + ".";
		
		Time -= secs;
		var f:Int = Math.floor(Time * 100);
		if (f < 0) f = 0;
		if (f < 10)
		{
			t += "0" + f;
		}
		else
		{
			t += f;
		}
		_best.text = t;
	}
	
	private var _secs00:FlxText;
	private var _secs0:FlxText;
	private var _frac00:FlxText;
	private var _frac0:FlxText;
	
	private var _total:FlxText;
	private var _last:FlxText;
	private var _best:FlxText;
	
}