package com.ludoko.ld48;

import flixel.effects.FlxTrail;
import flixel.FlxG;
import flixel.FlxObject;
import flixel.FlxSprite;
import flixel.FlxState;
import flixel.group.FlxGroup;
import flixel.util.FlxAngle;
import flixel.util.FlxMath;
import flixel.util.FlxPoint;
import flixel.util.FlxRandom;
import flixel.util.FlxRect;

class PlayState extends FlxState
{
	
	public var wave:Int = 1;
	public var maxCircles:Int = 3;
	
	override public function create():Void
	{
		G.playstate = this;
		
		// Set a background color
		FlxG.camera.bgColor = 0xfffdfdfd;
		
		// Show the mouse (in case it hasn't been disabled)
		#if !FLX_NO_MOUSE
		FlxG.mouse.hide();
		FlxG.mouse.useSystemCursor = true;
		#end
		
		super.create();
		
		
		G.gui = new GameGUI();
		
		_background = new Background();
		_timerBar = new TimerBar();
		
		// Set up groups
		_blocks = new FlxGroup();
		_fades = new FlxGroup();
		_circles = new FlxGroup();
		_circlesArr = new Array<TimeCircle>();
		_circleDec = new FlxGroup();
		_trails = new FlxGroup();
		_bullets = new FlxGroup();
		
		
		_circlePoints = new Array<CircleSpawn>();
		_circleTexts = new Array<CircleText>();
		addCircleSpawn(FlxG.width * 0.5, FlxG.height - 98, 96);
		addCircleSpawn(48, FlxG.height - 48, 64);
		addCircleSpawn(FlxG.width - 48, FlxG.height - 48, 64);
		addCircleSpawn(72, FlxG.height - 148, 64);
		addCircleSpawn(FlxG.width - 72, FlxG.height - 148, 64);
		addCircleSpawn(FlxG.width * 0.5, FlxG.height - 16, 64);
		addCircleSpawn(FlxG.width * 0.5, 40, 64);
		
		
		createBlock(-16, 0, 16, FlxG.height);
		createBlock(FlxG.width, 0, 16, FlxG.height);
		
		createBlock(0, -16, FlxG.width, 16);
		createBlock(0, FlxG.height - 16, FlxG.width, 16);
		
		
		createBlock(0, FlxG.height - 48, 96, 32);
		createBlock(FlxG.width - 96, FlxG.height - 48, 96, 32);
		
		createBlock(FlxG.width * 0.5 - 48, FlxG.height - 98, 96, 16);
		
		createBlock(48, FlxG.height - 148, 48, 16);
		createBlock(FlxG.width - 96, FlxG.height - 148, 48, 16);
		
		G.player = new Player(FlxG.width * 0.5 - 3, FlxG.height - 32);
		
		
		add(_background);
		add(_timerBar);
		add(_blocks);
		add(_fades);
		add(_circles);
		add(_circleDec);
		add(_trails);
		add(G.player);
		add(_bullets);
		add(G.gui);
		
		G.timer = 10; // 10 seconds!
		G.totalTime = 0;
		
		G.gui.updateLastTime(G.lastTime);
		G.gui.updateBestTime(G.bestTime);
	}
	
	public var blocks(get_blocks, never):FlxGroup;
	private function get_blocks():FlxGroup
	{
		return _blocks;
	}
	
	public var fades(get_fades, never):FlxGroup;
	private function get_fades():FlxGroup
	{
		return _fades;
	}
	
	public var trails(get_trails, never):FlxGroup;
	private function get_trails():FlxGroup
	{
		return _trails;
	}
	
	public var bullets(get_bullets, never):FlxGroup;
	private function get_bullets():FlxGroup
	{
		return _bullets;
	}
	
	public var circles(get_circles, never):Array<TimeCircle>;
	private function get_circles():Array<TimeCircle>
	{
		return _circlesArr;
	}
	
	
	public function addCircleSpawn(X:Float, Y:Float, Size:Int):Void
	{
		var c:CircleSpawn = new CircleSpawn(X, Y, Size);
		_circlePoints.push(c);
		
		var t:CircleText = new CircleText(X, Y);
		_circleDec.add(t);
		_circleTexts.push(t);
	}
	
	
	public function createBlock(X:Float, Y:Float, Width:Int, Height:Int):Block
	{
		var b:Block = new Block(X, Y, Width, Height);
		_blocks.add(b);
		return b;
	}
	
	public function addRandomCircle(X:Float, Y:Float, Size:Int, Types:Array<Int>):TimeCircle
	{
		var timeType:Int = Types[FlxRandom.intRanged(0, Types.length)];
		return addCircle(timeType, X, Y, Size);
	}
	
	public function addCircle(TimeType:Int, X:Float, Y:Float, Size:Int = 64):TimeCircle
	{
		if (_circlesArr.length >= maxCircles) return null;
		
		var c:TimeCircle = new TimeCircle(TimeType, X, Y, Size);
		_circles.add(c);
		_circlesArr.push(c);
		
		if (_circlesArr.length > maxCircles)
		{
			var oc:TimeCircle = _circlesArr.shift();
			_circles.remove(oc, true);
			oc.kill();
		}
		
		return c;
	}
	
	public function removeCircle(C:TimeCircle):Void
	{
		_circlesArr.remove(C);
		_circles.remove(C, true);
	}
	
	public function addTrail(Sprite:FlxSprite, Image:String):FlxTrail
	{
		var t:FlxTrail = new FlxTrail(Sprite, Image, 6, 3, 0.4, 0.05);
		_trails.add(t);
		return t;
	}
	
	public function addCircleFade(X:Float, Y:Float, Size:Int, Color:Int, InitialScale = 0, Speed:Float = 1, FadeSpeed:Float = 1):CircleFade
	{
		var f:CircleFade = new CircleFade(X, Y, Size, Color, InitialScale, Speed, FadeSpeed);
		_fades.add(f);
		return f;
	}
	
	public function randomBullet(BulletTypes:Array<Int> = null):Bullet
	{
		var bulletType = 0;
		if (BulletTypes == null)
		{
			bulletType = 0;
		}
		else
		{
			bulletType = BulletTypes[FlxRandom.intRanged(0, BulletTypes.length)];
		}
		
		var bx:Float = 0;
		var by:Float = 0;
		var vx:Float = 0;
		var vy:Float = 0;
		
		G.p1.x = G.player.centerX;
		G.p1.y = G.player.centerY;
		
		var s:Float = FlxRandom.intRanged(75, 120);
		
		var r:Float = FlxRandom.float();
		if (r < 0.3 && bulletType != 1)
		{
			if (FlxRandom.chanceRoll(50))
			{
				bx = G.player.centerX;
				by = -8;
				
				vx = 0;
				vy = s;
			}
			else 
			{
				by = G.player.centerY - 5;
				
				if (FlxRandom.chanceRoll(50))
				{
					bx = -8;
					vx = s;
				}
				else
				{
					bx = FlxG.width + 8;
					vx = -s;
				}
				vy = 0;
			}
		}
		else
		{
			if (FlxRandom.chanceRoll())
			{
				bx = FlxRandom.floatRanged(8, FlxG.width - 8);
				by = -8;
			}
			else
			{
				by = FlxRandom.floatRanged(8, FlxG.height - 8);
				if (FlxRandom.chanceRoll(50))
				{
					bx = -8;
				}
				else
				{
					bx = FlxG.width + 8;
				}
			}
			
			var ang:Float = Math.atan2(G.p1.y - by + FlxRandom.floatRanged(-8, 8), G.p1.x - bx + FlxRandom.floatRanged(-8, 8));
			vx = Math.cos(ang) * s;
			vy = Math.sin(ang) * s;
		}
		
		return addBullet(bulletType, bx, by, vx, vy);
	}
	
	public function addBullet(BulletType:Int, X:Float, Y:Float, VelX:Float = 0, VelY:Float = 0):Bullet
	{
		var b:Bullet = null;
		if (BulletType == 0)
		{
			b = new Bullet(X, Y, VelX, VelY);
		}
		else
		{
			b = new BounceBullet(X, Y, VelX, VelY);
		}
		_bullets.add(b);
		return b;
	}
	
	
	public function nextWave():Void
	{
		G.timer += 10;
		
		G.player.timeType++;
		if (G.player.timeType > 2)
		{
			G.player.timeType = 0;
		}
		
		var p:CircleSpawn;
		if (wave == 1)
		{
			maxCircles = 3;
			for (p in _circlePoints)
			{
				if (FlxRandom.chanceRoll(68))
				{
					addRandomCircle(p.x, p.y, p.size, [0]);
				}
			}
		}
		else if (wave == 2)
		{
			maxCircles = 2;
			for (p in _circlePoints)
			{
				if (FlxRandom.chanceRoll(68))
				{
					addRandomCircle(p.x, p.y, p.size, [1]);
				}
			}
		}
		else if (wave == 3 || wave == 4)
		{
			maxCircles = 3;
			for (p in _circlePoints)
			{
				if (FlxRandom.chanceRoll(68))
				{
					addRandomCircle(p.x, p.y, p.size, [0, 1]);
				}
			}
		}
		else if (wave == 5)
		{
			maxCircles = 2;
			for (p in _circlePoints)
			{
				if (FlxRandom.chanceRoll(68))
				{
					addRandomCircle(p.x, p.y, p.size, [2]);
				}
			}
		}
		else if (wave == 6 || wave == 7)
		{
			maxCircles = 3;
			for (p in _circlePoints)
			{
				if (FlxRandom.chanceRoll(68))
				{
					addRandomCircle(p.x, p.y, p.size, [0, 1, 2]);
				}
			}
		}
		else if (wave <= 10)
		{
			maxCircles = 4;
			for (p in _circlePoints)
			{
				if (FlxRandom.chanceRoll(68))
				{
					addRandomCircle(p.x, p.y, p.size, [0, 1, 2]);
				}
			}
		}
		else if (wave <= 14)
		{
			maxCircles = 5;
			for (p in _circlePoints)
			{
				if (FlxRandom.chanceRoll(68))
				{
					addRandomCircle(p.x, p.y, p.size, [0, 1, 2]);
				}
			}
		}
		else 
		{
			maxCircles = 6;
			for (p in _circlePoints)
			{
				if (FlxRandom.chanceRoll(68))
				{
					addRandomCircle(p.x, p.y, p.size, [0, 1, 2]);
				}
			}
		}
		
		_timerBar.resetTimer();
		
		wave++;
	}
	
	public function endGame():Void
	{
		_gameOver = true;
		
		G.player.kill();
		
		var newBest:Bool = false;
		
		G.lastTime = G.totalTime;
		if (G.totalTime > G.bestTime)
		{
			G.bestTime = G.totalTime;
			newBest = true;
		}
		
		add(new GameOverGUI(newBest));
	}

	
	override public function update():Void
	{
		if (_gameOver)
		{
			if (FlxG.keyboard.justPressed("Z"))
			{
				FlxG.switchState(new PlayState());
			}
			if (FlxG.keyboard.justPressed("ESCAPE"))
			{
				FlxG.switchState(new MenuState());
			}
			
			super.update();
			return;
		}
		
		/*if (FlxG.keyboard.justPressed("ONE"))
		{
			G.player.timeType = TimeCircle.TIME_SLOW;
		}
		if (FlxG.keyboard.justPressed("TWO"))
		{
			G.player.timeType = TimeCircle.TIME_FAST;
		}
		if (FlxG.keyboard.justPressed("THREE"))
		{
			G.player.timeType = TimeCircle.TIME_REVERSE;
		}*/
		
		if (FlxG.keyboard.justPressed("C"))
		{
			randomBullet();
		}
		
		// Calculate new time speed
		var newSpeed:Float = 1;
		
		G.p1.x = G.player.centerX;
		G.p1.y = G.player.centerY;
		
		var c:TimeCircle;
		for (c in _circlesArr)
		{
			G.p2.x = c.centerX;
			G.p2.y = c.centerY;
			var dist:Float = FlxMath.getDistance(G.p1, G.p2);
			
			var cscale:Float = c.getAdjustedTimeScale(dist);
			newSpeed *= cscale;
		}
		G.timeSpeed = newSpeed;
		
		
		_bulletTimer -= FlxG.elapsed;
		if (_bulletTimer <= 0)
		{
			if (wave == 1)
			{
				var l:Int = FlxRandom.intRanged(2, 3);
				for (i in 0 ... l)
				{
					randomBullet([0]);
				}
				_bulletTimer = FlxRandom.floatRanged(3.5, 4.5);
			}
			else if (wave == 2 || wave == 3)
			{
				var l:Int = FlxRandom.intRanged(3, 5);
				for (i in 0 ... l)
				{
					randomBullet([0]);
				}
				_bulletTimer = FlxRandom.floatRanged(3.25, 4.5);
			}
			else if (wave == 4)
			{
				var l:Int = FlxRandom.intRanged(3, 5);
				for (i in 0 ... l)
				{
					randomBullet([0, 1]);
				}
				_bulletTimer = FlxRandom.floatRanged(3.5, 4.5);
			}
			else if (wave == 5)
			{
				var l:Int = FlxRandom.intRanged(3, 5);
				for (i in 0 ... l)
				{
					randomBullet([0, 1]);
				}
				_bulletTimer = FlxRandom.floatRanged(3, 4.25);
			}
			else
			{
				var l:Int = FlxRandom.intRanged(4, 6);
				for (i in 0 ... l)
				{
					randomBullet([0, 1]);
				}
				_bulletTimer = FlxRandom.floatRanged(2, 4);
			}
		}
		
		
		super.update();
		
		if (newSpeed != 0)
		{
			var elapsed:Float = FlxG.elapsed * G.timeSpeed * G.absoluteTimeSpeed;
			G.timer -= elapsed;
			G.totalTime += elapsed;
			G.gui.updateTime(G.timer);
			G.gui.updateTotalTime(G.totalTime);
			
			if (G.timer <= 0)
			{
				nextWave();
			}
		}
		
		FlxG.collide(G.player, _blocks);
		
		G.r.x = G.player.centerX - 3;
		G.r.width = 6;
		if (!G.player.ducking)
		{
			G.r.y = G.player.centerY - 4;
			G.r.height = 8;
		}
		else 
		{
			G.r.y = G.player.centerY;
			G.r.height = 4;
		}
		FlxG.overlap(G.player, _bullets, bulletHitsPlayer);
		
		if (_gameOver)
		{
			endGame();
		}
	}
	
	private function bulletHitsPlayer(P:FlxObject, B:FlxObject):Void
	{
		if (_restart) return;
		
		var bullet:Bullet = cast(B, Bullet);
		G.p2.x = bullet.centerX;
		G.p2.y = bullet.centerY;
		
		if ((B.x + B.width > G.r.x) && (B.x < G.r.x + G.r.width) && (B.y + B.height > G.r.y) && (B.y < G.r.y + G.r.height))
		{
			_gameOver = true;
		}
	}
	
	override public function destroy():Void
	{
		super.destroy();
	}
	
	private var _restart:Bool = false;
	private var _gameOver:Bool;
	
	private var _background:Background;
	private var _timerBar:TimerBar;
	
	private var _blocks:FlxGroup;
	private var _fades:FlxGroup;
	private var _trails:FlxGroup;
	private var _circles:FlxGroup;
	private var _circlesArr:Array<TimeCircle>;
	private var _circleDec:FlxGroup;
	private var _bullets:FlxGroup;
	
	private var _bulletTimer:Float = 3;
	
	private var _circlePoints:Array<CircleSpawn>;
	private var _circleTexts:Array<CircleText>;
	
}