package com.ludoko.ld48;

import flixel.FlxG;
import flixel.FlxObject;
import flixel.FlxSprite;
import flixel.util.FlxMath;

/**
 * ...
 * @author Michael Lee
 */
class Player extends FlxSprite
{

	
	public var movespeed:Float = 700;
	public var gravity:Float = 600;
	public var minVelocityY:Float = 500;
	public var maxVelocityY:Float = 500;
	
	public var maxVelocityXNormal:Float = 100;
	public var maxVelocityXDash:Float = 160;
	public var maxVelocityYNormal:Float = 500;
	public var maxVelocityYWall:Float = 20;
	public var dragNormal:Float = 1500;
	public var dragDash:Float = 500;
	public var dragSlide:Float = 200;
	
	//public var dashTime:Float = 0.6;
	public var jumpVel:Float = 900;
	public var jumpLength:Float = 0.25;
	public var doubleJumpVel:Float = 600;
	public var doubleJumpLength:Float = 0.14;
	
	public var ducking:Bool = false;
	
	public var timeType:Int = 2;
	public var timeMultiplier:Float = 1;
	
	
	public function new(X:Float, Y:Float) 
	{
		super();
		moves = false;
		
		//makeGraphic(14, 14, 0xffff00ff);
		loadGraphic("assets/images/game/player.png", true, true, 16, 16);
		addAnimation("idle", [0], 0, false);
		addAnimation("run", [1], 0, false);
		addAnimation("dash", [12], 0, false);
		addAnimation("duck", [2], 0, false);
		addAnimation("jump", [10, 3, 3], 30, false);
		addAnimation("land", [10, 0], 12, false);
		addAnimation("flip", [4, 5, 6, 7, 8, 9, 4, 5, 6, 7, 8, 9, 0], 30, false);
		addAnimation("flip_reverse", [9, 8, 7, 6, 5, 4, 9, 8, 7, 6, 5, 4, 0], 30, false);
		addAnimation("flip_ground", [3, 4, 5, 6, 7, 8, 9, 4, 5, 6, 7, 8, 9, 0], 22, false);
		addAnimation("wall_grab", [11], 0, false);
		
		width = 6;
		height = 14;
		offset.x = 5;
		offset.y = 1;
		
		drag.x = dragNormal;
		maxVelocity.x = maxVelocityXNormal;
		minVelocityY = -maxVelocityYNormal;
		maxVelocityY = maxVelocityYNormal;
		
		reset(X, Y);
	}
	
	public var centerX(get_centerX, never):Float;
	public var centerY(get_centerY, never):Float;
	private function get_centerX():Float { return x + 3; }
	private function get_centerY():Float { return y + 7; }
	
	override public function reset(X:Float, Y:Float):Void 
	{
		super.reset(X, Y);
	}
	
	override public function update():Void 
	{
		acceleration.x = 0;
		acceleration.y = gravity;
		if (!_dashJump)
		{
			maxVelocity.x = maxVelocityXNormal;
		}
		minVelocityY = -maxVelocityYNormal;
		maxVelocityY = maxVelocityYNormal;
		
		var moving:Bool = false;
		var movingLeftRight:Bool = false;
		ducking = false;
		
		var prevOnGround:Bool = _onGround;
		_onGround = overlapsAt(x, y + 1, G.playstate.blocks);
		
		var touchingLeft:Bool = overlapsAt(x - 1, y, G.playstate.blocks);
		var touchingRight:Bool = overlapsAt(x + 1, y, G.playstate.blocks);
		
		if (!_onGround)
		{
			_dashing = false;
			_canDash = false;
			drag.x = dragNormal;
		}
		
		if (_onGround || touchingLeft || touchingRight)
		{
			_dashJump = false;
			_numJumps = 0;
		}
		
		var anim:String = "";
		var forceAnim:Bool = false;
		
		// For double-tap
		_lastButton -= FlxG.elapsed;
		
		_stopLeftRight -= FlxG.elapsed;
		if (_stopLeftRight <= 0)
		{
			var doubletap:Bool = false;
			
			if (FlxG.keyboard.pressed("LEFT"))
			{
				if (_lastMove != -1)
				{
					_dashing = false;
					_canDash = false;
					drag.x = dragNormal;
					_lastButton = 0;
				}
				if (_lastButton > 0 && _lastMove == -1)
				{
					_canDash = true;
					doubletap = true;
				}
				
				if (!touchingLeft)
				{
					facing = FlxObject.LEFT;
					acceleration.x = -movespeed;
					
					if (_onGround)
					{
						if (!doubletap && !_canDash)
						{
							anim = "run";
						}
						else
						{
							_dashing = true;
							maxVelocity.x = maxVelocityXDash;
							drag.x = dragDash;
							anim = "dash";
						}
					}
				}
				else
				{
					if (!_onGround)
					{
						anim = "wall_grab";
						//acceleration.y *= 0.25;
					}
					else
					{
						anim = "idle";
					}
					maxVelocityY = maxVelocityYWall;
				}
				
				_lastMove = -1;
				
				moving = true;
				movingLeftRight = true;
			}
			else if (FlxG.keyboard.pressed("RIGHT"))
			{
				if (_lastMove != 1)
				{
					_dashing = false;
					_canDash = false;
					drag.x = dragNormal;
					_lastButton = 0;
				}
				if (_lastButton > 0 && _lastMove == 1)
				{
					_canDash = true;
					doubletap = true;
				}
				
				if (!touchingRight)
				{
					facing = FlxObject.RIGHT;
					acceleration.x = movespeed;
					
					if (_onGround)
					{
						if (!doubletap && !_canDash)
						{
							anim = "run";
						}
						else
						{
							_dashing = true;
							maxVelocity.x = maxVelocityXDash;
							drag.x = dragDash;
							anim = "dash";
						}
					}
				}
				else
				{
					if (!_onGround)
					{
						anim = "wall_grab";
						//acceleration.y *= 0.25;
					}
					else
					{
						anim = "idle";
					}
					maxVelocityY = maxVelocityYWall;
				}
				
				_lastMove = 1;
				
				moving = true;
				movingLeftRight = true;
			}
			else if (FlxG.keyboard.justReleased("LEFT") || FlxG.keyboard.justReleased("RIGHT"))
			{
				_lastButton = 0.18;
			}
		}
		else
		{
			acceleration.x = _airAcc;
		}
		
		
		if (_onGround && !_jumping)
		{
			if (FlxG.keyboard.pressed("DOWN"))
			{
				ducking = true;
				acceleration.x = 0;
				if (_dashing)
				{
					drag.x = dragSlide;
				}
				anim = "duck";
				moving = true;
			}
		}
		
		if (_numJumps <= 0)
		{
			if (FlxG.keyboard.justPressed("Z") && (_onGround || touchingLeft || touchingRight) && !_jumping)
			{
				_jumping = true;
				_jumpTimer = 0;
				
				velocity.y = -110;
				minVelocityY = -maxVelocityYNormal;
				maxVelocityY = maxVelocityYNormal;
				
				if (!_onGround)
				{
					if (touchingLeft)
					{
						velocity.x = 150;
						_airAcc = 170;
						_stopLeftRight = 0.35;
					}
					else //if (touchingRight)
					{
						velocity.x = -150;
						_airAcc = -170;
						_stopLeftRight = 0.35;
					}
				}
				else
				{
					if (_dashing)
					{
						_dashJump = true;
					}
				}
				
				if (touchingLeft || touchingRight)
				{
					anim = "flip_reverse";
				}
				else if (!movingLeftRight)
				{
					anim = "jump";
				}
				else
				{
					anim = "flip_ground";
				}
				moving = true;
			}
			if (_jumping)
			{
				_jumpTimer += FlxG.elapsed;
				
				if (FlxG.keyboard.pressed("Z") && _jumpTimer < jumpLength)
				{
					if (finished && movingLeftRight)
					{
						anim = "flip";
					}
					
					velocity.y -= jumpVel * FlxG.elapsed; 
					moving = true;
				}
				else if (FlxG.keyboard.justReleased("Z") || _jumpTimer >= jumpLength)
				{
					_jumping = false;
					_numJumps++;
				}
			}
			
			if (FlxG.keyboard.justPressed("Z") && !_onGround && !touchingLeft && !touchingRight && !_jumping)
			{
				_jumping = true;
				_jumpTimer = 0;
				
				velocity.y = -160;
				
				anim = "flip";
				forceAnim = true;
				moving = true;
				
				_numJumps = 1;
			}
		}
		else if (_numJumps <= 1)
		{
			if (FlxG.keyboard.justPressed("Z"))
			{
				_jumping = true;
				_jumpTimer = 0;
				
				velocity.y = -160;
				
				anim = "flip";
				forceAnim = true;
				moving = true;
			}
			if (_jumping)
			{
				_jumpTimer += FlxG.elapsed;
				
				if (FlxG.keyboard.pressed("Z") && _jumpTimer < doubleJumpLength)
				{
					if (finished && movingLeftRight)
					{
						anim = "flip";
					}
					
					velocity.y -= doubleJumpVel * FlxG.elapsed; 
					moving = true;
				}
				else if (FlxG.keyboard.justReleased("Z") || _jumpTimer >= doubleJumpLength)
				{
					_jumping = false;
					_numJumps++;
				}
			}
		}
		
		if (!moving)
		{
			if (finished)
			{
				anim = "idle";
			}
		}
		
		if (!prevOnGround && _onGround)
		{
			anim = "land";
		}
		
		if (anim != "")
		{
			play(anim, forceAnim);
		}
		
		//if (FlxG.keyboard.justPressed("X"))
		//{
			//G.playstate.addCircle(timeType, centerX, centerY);
		//}
		
		super.update();
		updateMotionExt();
		
		if (velocity.x == 0)
		{
			_dashing = false;
			_canDash = false;
			drag.x = dragNormal;
		}
	}
	
	private function updateMotionExt():Void 
	{
		var delta:Float;
		var velocityDelta:Float;
		
		var dt:Float = FlxG.elapsed;
		
		velocityDelta = 0.5 * (FlxMath.computeVelocity(angularVelocity, angularAcceleration, angularDrag, maxAngular) - angularVelocity);
		angularVelocity += velocityDelta; 
		angle += angularVelocity * dt;
		angularVelocity += velocityDelta;
		
		velocityDelta = 0.5 * (FlxMath.computeVelocity(velocity.x, acceleration.x, drag.x, maxVelocity.x) - velocity.x);
		velocity.x += velocityDelta;
		delta = velocity.x * dt;
		velocity.x += velocityDelta;
		x += delta;
		
		velocityDelta = 0.5 * (FlxMath.computeVelocity(velocity.y, acceleration.y, drag.y, 0) - velocity.y);
		if (velocity.y > maxVelocityY)
		{
			velocity.y = maxVelocityY;
		}
		else if (velocity.y < minVelocityY)
		{
			velocity.y = minVelocityY;
		}
		velocity.y += velocityDelta;
		delta = velocity.y * dt;
		velocity.y += velocityDelta;
		y += delta;
	}
	
	override public function kill():Void 
	{
		super.kill();
	}
	
	private var _onGround:Bool = false;
	
	private var _lastButton:Float;
	private var _lastMove:Int = 0;
	private var _dashing:Bool = false;
	private var _canDash:Bool = false;
	private var _stopLeftRight:Float = 0;
	private var _airAcc:Float = 0;
	
	private var _jumping:Bool = false;
	private var _numJumps:Int = 0;
	private var _dashJump:Bool = false;
	private var _jumpTimer:Float = 0;
	
}