package com.ludoko.ld48;
import flixel.FlxG;
import flixel.FlxSprite;

/**
 * Time dilation circle
 * 
 * @author Michael Lee
 */
class TimeCircle extends FlxSprite
{

	public static inline var TIME_SLOW:Int = 0;
	public static inline var TIME_FAST:Int = 1;
	public static inline var TIME_REVERSE:Int = 2;
	public static inline var TIME_FREEZE:Int = 3;
	
	public static inline var MIN_DIST:Float = 14;
	
	public var tutorialMode:Bool = false;
	public var gameState:Dynamic;
	
	public var timeType:Int;
	public var radius:Int;
	public var diameter:Int;
	public var timeScale:Float = 0.95; // 0 is no time scale, 1 is max (i.e. full time stop)
	
	public function new(TimeType:Int, X:Float, Y:Float, Size:Int = 64) 
	{
		timeType = TimeType;
		
		gameState = G.playstate;
		
		_origRadius = Std.int(Size * 0.5);
		radius = 0;
		diameter = Size;
		_trueRadius = 0;
		
		var hs:Float = Size * 0.5;
		super(X - hs, Y - hs);
		
		if (Size == 64)
		{
			loadGraphic("assets/images/game/circle_64.png");
		}
		else if (Size == 96)
		{
			loadGraphic("assets/images/game/circle_96.png");
		}
		antialiasing = false;
		
		if (TimeType == TIME_SLOW)
		{
			color = 0xee94e7;
			timeScale = 0.85;
		}
		else if (TimeType == TIME_FAST)
		{
			color = 0xffd00a;
			timeScale = 0.75;
		}
		else if (TimeType == TIME_REVERSE)
		{
			color = 0xfa6e6e;
			timeScale = 2;
		}
		
		alpha = 0.32;
		setScale(0, true);
	}
	
	public var centerX(get_centerX, never):Float;
	public var centerY(get_centerY, never):Float;
	private function get_centerX():Float { return x + radius; }
	private function get_centerY():Float { return y + radius; }
	
	public function setScale(Scale:Float, Force:Bool = false):Void
	{
		if (_scale != Scale)
		{
			scale.x = scale.y = Scale;
		}
		_scale = Scale;
	}
	
	public function setRadius(Radius:Int):Void
	{
		radius = Radius;
		_trueRadius = radius + 2;
	}
	
	
	public function getAdjustedTimeScale(Dist:Float):Float
	{
		if (_state != 1 || Dist >= _trueRadius) return 1;
		
		if (timeType == TIME_SLOW)
		{
			if (Dist < MIN_DIST)
			{
				return (1 - timeScale);
			}
			var s:Float = ((Dist - MIN_DIST) / (_trueRadius - MIN_DIST)) * ((Dist - MIN_DIST) / (_trueRadius - MIN_DIST));
			return ((1 - timeScale) + (s * timeScale));
		}
		else if (timeType == TIME_FAST)
		{
			if (Dist < MIN_DIST)
			{
				return (1 + timeScale);
			}
			var s:Float = 1 - ((Dist - MIN_DIST) / (_trueRadius - MIN_DIST)) * ((Dist - MIN_DIST) / (_trueRadius - MIN_DIST));
			return (1 + (s * timeScale));
		}
		else if (timeType == TIME_REVERSE)
		{
			if (Dist < MIN_DIST)
			{
				return -timeScale;
			}
			var s:Float = 1 -((Dist - MIN_DIST) / (_trueRadius - MIN_DIST)) * ((Dist - MIN_DIST) / (_trueRadius - MIN_DIST));
			return -s;
		}
		
		return 1;
	}
	
	override public function update():Void 
	{
		if (_state == 0)
		{
			var newScale:Float = _scale;
			newScale += 5 * FlxG.elapsed;
			
			var r:Int = Std.int(newScale * _origRadius);
			setRadius(r);
			
			if (radius >= _origRadius)
			{
				_state = 1;
				newScale = 1;
				setRadius(_origRadius);
				
				_timer = 9.7;
			}
			setScale(newScale);
		}
		else if (_state == 1)
		{
			_fadeTimer -= FlxG.elapsed;
			if (_fadeTimer <= 0)
			{
				if (timeType == TIME_SLOW)
				{
					gameState.addCircleFade(centerX, centerY, diameter, color, 1, -0.333, 0.333);
					_fadeTimer += 0.75;
				}
				else if (timeType == TIME_FAST)
				{
					gameState.addCircleFade(centerX, centerY, diameter, color, 1, -1, 1);
					_fadeTimer += 0.333;
				}
				else if (timeType == TIME_REVERSE)
				{
					gameState.addCircleFade(centerX, centerY, diameter, color, 0, 0.5, 0.5);
					_fadeTimer += 0.65;
				}
			}
			
			if (!tutorialMode)
			{
				_timer -= FlxG.elapsed * G.timeSpeed * G.absoluteTimeSpeed;
				if (_timer <= 0)
				{
					_state = 2;
				}
			}
		}
		else if (_state == 2)
		{
			var newScale:Float = _scale;
			newScale -= 10 * FlxG.elapsed;
			
			var r:Int = Std.int(newScale * _origRadius);
			setRadius(r);
			
			if (radius <= 0)
			{
				kill();
			}
			else
			{
				setScale(newScale);
			}
		}
		
		super.update();
	}
	
	override public function kill():Void 
	{
		super.kill();
		G.playstate.removeCircle(this);
	}
	
	private var _state:Int = 0;
	private var _scale:Float = 1;
	private var _timer:Float = 0;
	
	private var _origRadius:Int;
	private var _trueRadius:Int;
	
	private var _fadeTimer:Float = 0.333;
	
}